###############################################################################
# db_spaces.awk
# 
# Changes:
# 03/24/1999 Baer
#	Changed code to properly calculate space utilization for 
#	blobspaces.
###############################################################################
# -----------------------------------------------------------------------------
function disk_bal()
# -----------------------------------------------------------------------------

{
  # 
  # Cleanup (in case it didn't happen last time)
  #

  function_cleanup()

  # 
  # First, run onstat -D to get the largest read value (for graph scale)
  #

  system("onstat -D | grep \" /\" | sort -bnr +4 > "tmpfl2)
  
  raw = "===================================="
  getline < tmpfl2
 

  scale = ($5 / length(raw)) * 2
  close tmpfl2 

  if (scale == 0)
  {
    printf("db_spaces | Error: scale value == 0\n")
    system("cat "tmpfl2)
    return 1
  }

  # 
  # Gather read data
  #

  system("onstat -D | grep \" /\" | sort -b +6 > "tmpfl2)

  flag = 0
  stop = 0

  msg = sprintf("Disk I/O Distribution Usage Report\n")
  print msg 

  printf(" Disk Path                                    Reads    Writes     Total\n") 
  printf(" ---------------------------------------- --------- --------- ---------\n") 

  last_disk = ""

  t_reads  = 0
  t_writes = 0
  t_total  = 0

  while (getline < tmpfl2 > 0 && stop == 0)
  {
    if (NF != 7)
    {
      printf("db_spaces | Error: onstat -D output doesn't contain 7 fields\n")
      system("cat "tmpfl2)
      return 1
    }

    if ($7 != last_disk)
    {
      if (last_disk != "")
      {
        d_graph = substr(raw,1,d_total / scale) ">"
        printf(" %-40s %9i %9i %9i %s\n",last_disk,d_reads,d_writes,d_total,d_graph) 
      }

      last_disk = $7
      d_reads   = $5
      d_writes  = $6
      d_total   = d_reads + d_writes

    }
    else
    {
      d_reads   += $5
      d_writes  += $6
      d_total   = d_reads + d_writes
    } 

    t_reads  += $5
    t_writes += $6
    t_total   = t_reads + t_writes

  }
  close tmpfl2 

  if (scale == 0) scale = d_total

  d_graph = substr(raw,1,d_total / scale) ">"
  printf(" %-40s %9i %9i %9i %s\n",$7,d_reads,d_writes,d_total,d_graph) 

  last_disk = "" 
  printf("                                          ========= ========= =========\n") 
  printf(" %-40s %9i %9i %9i\n",last_disk,t_reads,t_writes,t_total) 
  printf("\n") 
  
  #system("cat "tmpfl2)
  #printf("db_spaces | disk_bal Done\n")

}


# -----------------------------------------------------------------------------
function stat_d()
# -----------------------------------------------------------------------------

{

  function_cleanup()

  if (system("onstat -d > "tmpfl2) != 0)     # Run Informix "onstat -d"
  {                                          # Did it complete successfully?
    printf("db_spaces | onstat error\n")     # No! print error message.
    exit 1				     # Get outta here 
  }
  
  flag = 0
  stop = 0				     # Flag to identify when to quit

  max_dbsp = 0				     # Init the var. for total dbspaces
  num_lines = 0				     # Init the var. for total lines

  #
  # Delete previous variables
  #

  for (x in dbsp) delete dbsp[x]
  for (x in dbspsize) delete dbspsize[x]
  for (x in dbspfree) delete dbspfree[x]
  for (x in dbspused) delete dbspused[x]
  for (x in dbspchks) delete dbspchks[x]
  for (x in chunk) delete chunk[x]  
  for (x in dbsnum) delete dbsnum[x]
  for (x in offset) delete offset[x] 
  for (x in path) delete path[x]  
  for (x in size) delete size[x]  
  for (x in free) delete free[x] 
  for (x in thru) delete thru[x]
  for (x in dbspsize) delete dbspsize[x] 
  for (x in dbspfree) delete dbspfree[x] 
  for (x in dbspchks) delete dbspchks[x] 
  for (x in dbspused) delete dbspused[x]
  for (x in dbsppctu) delete dbsppctu[x] 
  for (x in pctused) delete pctused[x]

  #
  # get list of dbspaces
  #

  while (getline < tmpfl2 > 0 && stop == 0)
  {
    if ($2 == "active,") 		    # Did we find end of dbspace list
    {					    # Yes
      stop = 1 				    # Set flag to stop processing
      flag = 0 
    }

    if (flag == 1)			    # Are we processing dbspaces?
    {   				    # Yes
      dbsp[$2] = $NF			    # Store dbspace name
      dbspsize[$2] = 0			    # Init dbspace size field
      dbspfree[$2] = 0			    # Init dbspace free field
      dbspused[$2] = 0			    # Init dbspace used field
      dbspchks[$2] = 0			    # Init number of chunks field	
	
      if ($2 > max_dbsp) 		    # Are we looking at dbspace lines?
      {					    # Yes
	max_dbsp = $2			    # MAX number of dbspaces=dbspace num
      }					    # Done "if ($2"
    }					    # Done "if (flag"


    if ($1 == "address")		    # Start processing dbspaces?
    {					    # Yes, "onstat -d" first line read	
	flag = 1			    # Set "dbspace" process flag on
    }					    # Done "if"

  }
   
  close tmpfl2				    # Close file containing "onstat -d"

  if (debug) printf("debug: max_dbsp=%s\n",max_dbsp) 

  #######################################################################
  # Get disk space utilization data for dbspace chunks
  #######################################################################

  system("onstat -d | grep [PM]O[-B]  > "tmpfl1) # Get chunk data 

  i=0					    # Init chunk counter
  flag=0

  while (getline < tmpfl1 > 0)		    # Read a line of "onstat -d" output
  {				
    chunk[i]  = $2  	   		    # Store chunk number in array
    dbsnum[i] = $3 	   		    # Store dbspace number in array 
    offset[i] = $4 	   		    # Store chunk offset in array 
    size[i]   = $5			    # Store chunk size in array
    free[i]   = $6			    # Store chunk free space in array
    thru[i]   = offset[i] + size[i] - 1     # Store chunk ending offset
    if (NF == 8)			    # Are there 8 fields on each line? 
    {					    # Yes!  Looking at dbspace chunk
      path[i]   = $8			    # Store chunk path in array
      flags[i]  = substr($7,1,1)	    # Store first chunk flag (P or M)
    }
    else
    {					    # Looking at blobspace chunk
      path[i]   = $9			    # Store chunk path in array
      flags[i]  = substr($8,1,1)	    # Store first chunk flag (P or M)
      free[i]  = substr(free[i],2) # Get rid of ~ in blobspace
    }

    if (flags[i] == "P")		    # Is this a "primary" chunk
    {
        dbspsize[$3] += size[i]		    # Add chunk size to dbspace total
	dbspfree[$3] += free[i]		    # Add chunk free to dbspace total
	dbspchks[$3] ++ 		    # Add chunk count to dbspace total
    }					    # Done "if (flags"

    if (debug) print "dbsp=" dbsp[$3] " " dbspsize[$3] " " dbspfree[$3] " " dbspchks[$3]

    if (size[i] == 0)			    # Is chunk size zero?
    {					    # Yes, sompin went wrong 
        printf("db_spaces | zero divide #4\n") # Prepare error messages
        printf("db_spaces | dbsp=\n",dbsp[i])  # indicating dbspace with zero	
        system("cat "tmpfl1)		       # chunk size.
        msg = sprintf("db_spaces | zero divide #2\n")
        printf(msg)
        return 1			    # Get outta here
    }					    # Done "if (size"

    pctused[i] = (size[i]-free[i])/size[i]*100 # Calculate percent used

    gb_warn = 0

    i++				    # Increment chunk counter

  }					    # Done "while (getline

  close tmpfl1				    # Close the  chunk report

  num_lines = i-1

  for (i=0; i<= max_dbsp; i++)
  {

    if (dbsp[i] == "") continue

    if (dbspsize[i] == 0)
    {
      printf("dbamon | "title[thishost]" | zero divide #6\n")
      dbspsize[i] = 1
    }

    dbspused[i] = (dbspsize[i]-dbspfree[i])
    dbsppctu[i] = (dbspsize[i]-dbspfree[i])/dbspsize[i]*100
  }

  #
  # Create CHUNK Disk Space Report 
  #

  if (level > 1)
  {
    msg = sprintf("Disk Usage Report\n") 
    print msg 

    msg = sprintf("Chunk Report (Units: MB - Sorted by Device)\n") 
    print msg

    msg = sprintf("       DBSpace        P/M Chunk Unix Device                               Offset     End    Size    Free    Used  Pct")
    print msg 
    msg = sprintf("       -------------- --- ----- --------------------------------------- -------- ------- ------- ------- ------- ----")
    print msg

    total_free = 0
    total_size = 0
    total_used = 0

    system("rm " tmpflt " > /dev/null 2>&1")

    for (i=0; i<= num_lines; i++)
    {
      comment = ""


      used[i] = size[i] - free[i]

      if (pctused[i] < 1) 
      {
        #print "size=" size[i] " free=" free[i]
        comment = "Empty?"
      }

      msg = sprintf(" %-14s %-3s %5s %-40s %7d %7d %7d %7d %7d %3.0f%% %s",dbsp[dbsnum[i]],flags[i],chunk[i],path[i],offset[i]/488.28,thru[i]/488.28,size[i]/488.28,free[i]/488.28,used[i]/488.28,pctused[i],comment)
      print msg >> tmpflt

      total_free += free[i] 
      total_size += size[i] 
      total_used += used[i]  
    }    

    if (total_size == 0)
    {
      printf("db_spaces | zero divide #1\n")
      return 1
    }

    close tmpflt
    system("sort +3 +4 " tmpflt)

    msg = sprintf("                                                                                   ======= ======= ======= ====")
    print msg 

    msg = sprintf("                                                                                   %7d %7d %7d %3.0f%%",total_size/488.28,total_free/488.28,total_used/488.28,total_used/total_size*100)
    print msg 
  }
  
#  
# Create DBSPACE Report
#

  msg = sprintf("\n      DBSpace Report (Units: Million Bytes)\n")
  print msg 

  msg = sprintf("      DBSpace           Chunks       Size       Free       Used   Pct Full")
  print msg 
  msg = sprintf("      ----------------- ------ ---------- ---------- ---------- ----------")
  print msg 

  total_size = 0
  total_free = 0
  total_used = 0
  total_chks = 0

  system("rm " tmpflt " > /dev/null 2>&1")

  for (i=0; i<= max_dbsp; i++)
  {

    if (dbsp[i] != "")
    {
      #if (dbspsize[i]/488.28 < 1) dbspsize[i] = 488.28
      #if (dbspfree[i]/488.28 < 1) dbspfree[i] = 488.28
      #if (dbspused[i]/488.28 < 1) dbspused[i] = 488.28

      pre  = substr(dbsp[i],1,3)
      if (dbsppctu[i] >= 90 && pre != "log" && pre != "phy" )
      {
        comment = ">=90%"
      }
      else
      {
        comment = ""
      } 

      msg = sprintf("THIS: %-20s %3s  %9.1d  %9.1d  %9.1d    %6.0f  %s",dbsp[i],dbspchks[i],dbspsize[i]/488.28,dbspfree[i]/488.28,dbspused[i]/488.28,dbsppctu[i],comment)
      print msg >> tmpflt

      total_size += dbspsize[i]/488.28
      total_free += dbspfree[i]/488.28
      total_used += dbspused[i]/488.28
      total_chks += dbspchks[i]
    }

  }

  close tmpflt
  system("sort " tmpflt)

  msg = sprintf("                        ====== ========== ========== ========== ==========")
  print msg 

  if (total_size == 0)
  {
    printf("db_spaces | zero divide #5\n")
    return 1
  }

  msg = sprintf("      Total:        %10d %10d %10d %10d %9.0f%% \n",total_chks,total_size,total_free,total_used,total_used/total_size*100)
  print msg 

  grand_total_size += total_size / 1000

#  
# Create Disk I/O Balance Report
#

  if (level > 2) 
  {
    disk_bal()
  }

}
#  
# Clean up
#

function_cleanup()
{
  close report_path 
}


# -----------------------------------------------------------------------------
function function_cleanup()
# -----------------------------------------------------------------------------
{
  system("rm /tmp/db_spaces_tmp* > /dev/null 2>&1")
}


# -----------------------------------------------------------------------
# MAIN Logic
# -----------------------------------------------------------------------
{

  #
  # debug flags
  #

  debug      = 0
  debug_html = 0
  debug_read = 0
  debug_opc  = 0
  debug_crit = 0

  #
  # set other variables
  #

  # Cleanup old temp files
  system("rm /tmp/db_spaces* > /dev/null 2>&1")

  tmpfl1="/tmp/db_spaces1"
  tmpfl2="/tmp/db_spaces2"
  tmpflt="/tmp/db_spacest"

  if (length(level) == 0) level = 1
  #
  # Check disk usage
  #
  stat_d()

  # Cleanup temp files
  system("rm /tmp/db_spaces* > /dev/null 2>&1")

  exit
}

